/******************************************************************************//**
 * \file wanpipe_logger.h
 * \brief WANPIPE(tm) Wanpipe Logger API Headers and Defines
 *
 * Authors: David Rokhvarg <davidr@sangoma.com>
 *
 * Copyright (c) 2007 - 09, Sangoma Technologies
 * All rights reserved.
 *
 * * Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Sangoma Technologies nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Sangoma Technologies ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Sangoma Technologies BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * ===============================================================================
 */

#ifndef __WANPIPE_LOGGER_API_HDR__
#define __WANPIPE_LOGGER_API_HDR__

#if defined(__WINDOWS__)
# include "wanpipe_ctypes.h"
#endif

/***********************************************//**
 Wanpipe Logger Event Structure
****************************************************/

/*!
  \def WP_MAX_NO_BYTES_IN_LOGGER_EVENT
  \brief Maximum length of event string
 */
#define WP_MAX_NO_BYTES_IN_LOGGER_EVENT	256

/*!
  \def WP_MAX_NO_LOGGER_EVENTS
  \brief Maximum number of logger events kept by API driver
 */
#define WP_MAX_NO_LOGGER_EVENTS	512

/*!
 \struct wp_logger_event
 \brief Wanpipe API Logger Event contains event type,
		timestamp and data.
 
 \typedef wp_logger_event_t
 \brief Wanpipe API Logger Event contains event type,
		timestamp and data.
*/
typedef struct wp_logger_event{

	u_int32_t	event_type;		/*!< A bitmap indicating type of the event. Only a single bit will be set by API. */
	time_t	time_stamp_sec;		/*!< Timestamp in seconds (Since January 1, 1970). */
	long_t	time_stamp_usec;	/*!< microseconds portion of the timestamp */
	unsigned char	data[WP_MAX_NO_BYTES_IN_LOGGER_EVENT];	/*!< data area containing the logger event null-terminated string */

}wp_logger_event_t;

/* Definitions for 'event_type' in wp_logger_event_t: */
/* General purpose Wanpipe Logger event types. */
#define SANG_LOGGER_INFORMATION		(1)
#define SANG_LOGGER_WARNING			(1 << 2)
#define SANG_LOGGER_ERROR			(1 << 3)

/* Task-specific Wanpipe Logger event types - for advanced debugging only. */
#define SANG_LOGGER_TE1				(1 << 4)
#define SANG_LOGGER_HW_ECHO_CANCELLER			(1 << 5)
#define SANG_LOGGER_HW_ECHO_CANCELLER_VERBOSE1	(1 << 6)
#define SANG_LOGGER_HW_ECHO_CANCELLER_VERBOSE2	(1 << 7)
#define SANG_LOGGER_TDMAPI			(1 << 8)
#define SANG_LOGGER_FRONT_END		(1 << 9)
#define SANG_LOGGER_BRI				(1 << 10)


#define SANG_DECODE_LOGGER_EVENT_TYPE(bit)	\
	(bit & SANG_LOGGER_INFORMATION) ?	"SANG_LOGGER_INFORMATION":	\
	(bit & SANG_LOGGER_WARNING)		?	"SANG_LOGGER_WARNING"	:	\
	(bit & SANG_LOGGER_ERROR)		?	"SANG_LOGGER_ERROR"	: \
	(bit & SANG_LOGGER_TE1)			?	"SANG_LOGGER_TE1"	: \
	(bit & SANG_LOGGER_HW_ECHO_CANCELLER)	? "SANG_LOGGER_HW_ECHO_CANCELLER"	: \
	(bit & SANG_LOGGER_HW_ECHO_CANCELLER_VERBOSE1)	? "SANG_LOGGER_HW_ECHO_CANCELLER_VERBOSE1"	: \
	(bit & SANG_LOGGER_HW_ECHO_CANCELLER_VERBOSE2)	? "SANG_LOGGER_HW_ECHO_CANCELLER_VERBOSE2"	: \
	(bit & SANG_LOGGER_TDMAPI)		? "SANG_LOGGER_TDMAPI"	: \
	(bit & SANG_LOGGER_FRONT_END)	? "SANG_LOGGER_FRONT_END"	: \
	(bit & SANG_LOGGER_BRI)			? "SANG_LOGGER_BRI"	: \
	"Invalid Bit for Logger"



typedef struct wp_logger_stats
{
	ulong_t rx_events;			/*!< total Events received    */
	ulong_t rx_events_dropped;	/*!<  number of Events discarded because no free buffer available */
	ulong_t	max_event_queue_length;/*!<  maximum number of Events which can be stored in API queue */
	ulong_t	current_number_of_events_in_event_queue;/*!< number of Events currently in API queue */

}wp_logger_stats_t;

/*!
  \struct wp_logger_cmd
  \brief Wanpipe Logger API Command Structure used with WANPIPE_IOCTL_LOGGER_CMD

  Wanpipe Logger API Command structure used to execute WANPIPE_IOCTL_LOGGER_CMD commands
  All commands are defined in: 
  enum wp_logger_cmds

  \typedef wp_logger_cmd_t
 */
typedef struct wp_logger_cmd
{
	unsigned int cmd;		/*!< Command defined in enum wp_logger_cmds */
	unsigned int result;	/*!< Result defined in: enum SANG_STATUS or SANG_STATUS_T */

	union {
		wp_logger_event_t logger_event;	/*!< Wanpipe Logger Event */
		u_int32_t	logger_level;		/*!< A bitmap indicating types of the events logged by Wanpipe Logger. */
		u_int32_t	open_cnt;			/*!< Number of Open file descriptors for the device */
		wp_logger_stats_t stats;		/*!< Wanpipe Logger API Statistics */
	};

}wp_logger_cmd_t;

/*!
  \enum wanpipe_api_cmds
  \brief Commands used with WANPIPE_IOCTL_LOGGER_CMD IOCTL
 */
enum wp_logger_cmds
{
	WP_API_LOGGER_CMD_FLUSH_BUFFERS,	/*!< Flush Buffers */
	WP_API_LOGGER_CMD_READ_EVENT,		/*!< Read Logger Event */

	WP_API_LOGGER_CMD_GET_STATS,		/*!< Get device statistics */
	WP_API_LOGGER_CMD_RESET_STATS,		/*!< Reset device statistics */
	WP_API_LOGGER_CMD_OPEN_CNT,			/*!< Get Number of Open file descriptors for the device */

	WP_API_LOGGER_CMD_GET_LOGGER_LEVEL,	/*!< Get current level (types of events) of Wanpipe Logger */
	WP_API_LOGGER_CMD_SET_LOGGER_LEVEL	/*!< Set current level (types of events) of Wanpipe Logger */
};

#endif/* __WANPIPE_LOGGER_API_HDR__ */
